@@ -6,7 +6,7 @@
 use v5.8;
 
 package Parallel::WorkUnit;
-$Parallel::WorkUnit::VERSION = '1.005';
+$Parallel::WorkUnit::VERSION = '1.006';
 # ABSTRACT: Provide easy-to-use forking with ability to pass back data
 
 use strict;
@@ -123,10 +123,18 @@ sub waitall {
     if ( $#_ != 0 ) { confess 'invalid call'; }
     my ($self) = @_;
 
-    my $sp = $self->_subprocs();
-    if ( !keys(%$sp) ) { return; }
+    # Tail recursion
+    if ( $self->waitone() ) { goto &waitall }
+}
 
 
+sub waitone {
+    if ( $#_ != 0 ) { confess 'invalid call'; }
+    my ($self) = @_;
+    
+    my $sp = $self->_subprocs();
+    if ( !keys(%$sp) ) { return undef; }
+
     if ($do_thread) {
         my $child = $self->_queue()->dequeue(1);
 
@@ -148,14 +156,16 @@ sub waitall {
                         $self->_read_result($child);
 
                         waitpid($child, 0);
+
+                        return 1;  # We don't want to read more than one!
                     }
                 }
             }
         }
     }
-   
-    # Tail recursion
-    goto &waitall;
+
+    # We should never get here
+    return undef;
 }
 
 
@@ -182,6 +192,15 @@ sub wait {
     return $result;
 }
 
+
+sub count {
+    if ( $#_ != 0 ) { confess 'invalid call'; }
+    my ($self) = @_;
+    
+    my $sp = $self->_subprocs();
+    return scalar(keys %$sp);
+}
+
 sub _send_result {
     if ( $#_ != 3 ) { confess 'invalid call'; }
     my ( $self, $fh, $msg, $pid ) = @_;
@@ -283,7 +302,7 @@ Parallel::WorkUnit - Provide easy-to-use forking with ability to pass back data
 
 =head1 VERSION
 
-version 1.005
+version 1.006
 
 =head1 SYNOPSIS
 
@@ -348,10 +367,19 @@ method will return.
 If a child dies unexpectedly, this method will C<die()> and propagate a
 modified exception.
 
+=head2 waitone()
+
+This method similarly to C<waitall()>, but only waits for
+a single PID.  It will return after any PID exits.
+
+If this method is called when there is no processes executing,
+it will simply return undef. Otherwise, it will wait and then
+return 1.
+
 =head2 wait($pid)
 
-This functions simiarly to C<waitall()>, but only waits for
-a single PID.  See the C<waitall()> documentation above
+This functions simiarly to C<waitone()>, but waits only for
+a specific PID.  See the C<waitone()> documentation above
 for details.
 
 If C<wait()> is called on a process that is already done
@@ -359,6 +387,12 @@ executing, it simply returns.  Otherwise, it waits until the
 child process's work unit is complete and executes the callback
 routine, then returns.
 
+=head2 count()
+
+This method returns the number of currently outstanding
+threads (in either a running state or a waiting to send their
+output).
+
 =head1 BUGS
 
 Windows doesn't do C<fork()>, but emulates it with threads.  As a result,
@@ -375,7 +409,7 @@ Joel Maslak <jmaslak@antelope.net>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2015 by Joel Maslak.
+This software is copyright (c) 2015,2016 by Joel Maslak.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
